/*
==============================================================================
COPYRIGHT(C) 2001 Sergei Hristov Kostov            Bulgaria       city:Razgrad 

date:25.07.2001

 This file is part of mplay free software; you can redistribute it
 and/or modify it under the terms of the GNU General Public License as
 published by the Free Software Foundation; either version 2 of the
 License, or any later version.
       
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
       
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

==============================================================================
*/
#include <mikmod.h>
#include <unistd.h>
#include <curses.h>
#include <math.h>
#include "vroutines.cpp"
#define __PREV   'a'
#define __NEXT   'z'
#define __QUIT   'Q'
#define __VOUP   ']'
#define __VODW   '['
#define __PAUSE  'p'

MODULE *module;
WINDOW *win,*infowin,*dwin,*hintwin,*volwin;
DIR *curdir;
int nfiles=0,quit=0,next=0,prev=0;
const double grad=6.5;

 class CursInterface
      {
       public :
	 void interface (void);
	 void whoami (void);
	 void info (WINDOW *where);
	 void mhint (WINDOW *hwin);
      };
	 
 class DirList:public Linear_list
	{
	 private:
		int w;
	 public :
	   void showfiles(WINDOW *dirwin,int dx,int dy,int nlines);
	};
	
 class MikModPlayer
      {
      private :
	   int volume,m,s,sold,key,done;
      public :
	void init();
	int play(char *song,int curios,CursInterface screena);
	void timer(WINDOW *wheretimer, int tx,int ty);
	void keyboard();
	void vol(WINDOW *wh);
	void endprog();
      };

void CursInterface :: interface(void)
 {
  initscr();start_color();cbreak();
  noecho();clear();refresh();
  timeout(0);
  win = newwin(24,80,1,0);
  wborder(win,0,0,0,0,0,0,0,0);
  wrefresh(win);
  infowin=subwin(win,10,30,2,2);
  wborder(infowin,0,0,0,0,0,0,0,0);
  wrefresh(infowin);
  dwin=subwin(win,22,40,2,38);
  wborder(dwin,0,0,0,0,0,0,0,0);
  wrefresh(dwin);
  hintwin=subwin(win,11,30,13,2);
  wborder(hintwin,0,0,0,0,0,0,0,0);
  wrefresh(hintwin);
  volwin=subwin(win,22,6,2,32);
  wborder(volwin,0,0,0,0,0,0,0,0);
  wrefresh(volwin);
 };

void CursInterface :: whoami()
 {
  WINDOW *win1;
  win1=newwin(1,80,0,0);
  waddstr(win1," Linux Mod Player - 010723                               Sergei H. Kostov  .BG");
  wrefresh(win1);
  delwin(win1);
 };

void CursInterface :: info(WINDOW *where) 
{
int hx=2,hy=1;
  wclear(where);
  wborder(where,0,0,0,0,0,0,0,0);
  mvwprintw(where,hy++,hx,"########## info ##########");
  mvwprintw(where,hy++,hx," Title   : %s",module->songname);
  mvwaddstr(where,hy++,hx," Comment : ");
  if (module->comment==NULL)
               {waddstr(where,"No Comment ");}
	                      else waddstr(where,module->comment);
  mvwprintw(where,hy++,hx," Channels: %d",module->numchn);
  mvwprintw(where,hy++,hx," Samples : %d",module->numsmp);
  mvwprintw(where,hy++,hx," Voices  : %d",module->numvoices);
  mvwprintw(where,hy++,hx," Tracer  : %s",module->modtype);
  mvwaddstr(where,hy++,hx," Time    :");
  wrefresh(where);
};
  
void CursInterface :: mhint (WINDOW *hwin)
{
int startx=2,starty=2;
  mvwprintw(hwin,1,startx," ###### hint line ###### ");starty++;
  mvwprintw(hwin,starty++,startx," ~%c~  ~%c~ <=> volume     ",__VODW,__VOUP);
  mvwprintw(hwin,starty++,startx," Press ~%c~  Previos song ",__PREV);
  mvwprintw(hwin,starty++,startx," Press ~%c~  Next song ",__NEXT);
  mvwprintw(hwin,starty++,startx," Press ~%c~ (Un)Pause song ",__PAUSE);
  mvwprintw(hwin,starty++,startx," Press ~%c~  exit()    ",__QUIT);
  wrefresh(hwin);
};


void DirList::showfiles(WINDOW *dirwin,int dx,int dy,int nlines)
{
  L_list_To_first();
  w=0;
  nlines--;
  mvwprintw(dirwin,dy++,dx,"########### Song Container ###########");
  while(w!=nlines)
          {
	   w++;
           mvwprintw(dirwin,dy++,dx,"%s",L_names->name);
           if (L_list_GoTo_next()<0) break;
	  };  
   wrefresh(dirwin);
   closedir(curdir);
  L_list_To_first(); // da se vurnem v izhodna pozijiq tazi funcjiq trqbva da e napulno prozra4na ? 
};

void MikModPlayer :: init()
{
MikMod_RegisterAllDrivers();
MikMod_RegisterAllLoaders();
md_mode|=DMODE_SOFT_MUSIC;
volume=100;
m=0;s=0;sold=0;done=0;
if(MikMod_Init())
  {
   fprintf(stderr,"Could not initialize sound, reason: %s\n",MikMod_strerror(MikMod_errno));
   return;
  };
};

int MikModPlayer :: play(char *song,int curios,CursInterface screena)
{
int timex,timey;
 module = Player_Load(song,128,curios);
 if (module)
     {
      Player_Start(module);
      screena.info(infowin);
      getyx(infowin,timey,timex);timex++;
   while(done!=1 && Player_Active())
         {
	  timer(infowin,timex,timey);
	  keyboard();
	  vol(volwin);
	  Player_SetVolume(volume);
	  MikMod_Update();
         };
   Player_Stop();
   Player_Free(module);
   s=0,m=0,sold=0;/* zero timer O;-P*/
   }
  else
 {return -1;};
};

void MikModPlayer :: timer(WINDOW *wheretimer, int tx,int ty)
{

s=((module->sngtime)/1000);
if ( s > (sold+59) )
       {
	   m++;
	   sold=s;
        };
 mvwprintw(wheretimer,ty,tx,"%d:%d ",m,(s-sold));
 wrefresh(wheretimer);
};

void MikModPlayer :: keyboard()
{
key=getch();
  switch ((char)key)
                   {
		        case __VOUP: if(volume<128)volume++;break;
		        case __VODW: if(volume>0)volume--;break;
		        case __PREV: 
				{Player_Stop();
				 prev=1;
				}break;
		        case __NEXT: 
				{Player_Stop();
				 next=1;
				} break;
	                case __PAUSE:Player_TogglePause();break;
 	                case __QUIT: {done=1;endprog();quit=1;};break;
                    }; 
};

void MikModPlayer :: vol(WINDOW * wh)
{
int i,ty=20,tx=2;
ty=20;tx=1;
double howmuch;

for (i=19;i>1;i--)
{mvwprintw(wh,i,tx,"    ");
};
howmuch=(volume/grad);
for (i=0;i<(int)howmuch;i++)
{
mvwprintw(wh,ty--,tx,"||||");
};
wrefresh(wh);
};

void MikModPlayer :: endprog()
{
 delwin(win);
 delwin(infowin);
 delwin(dwin);
 delwin(hintwin);
 MikMod_Exit();
 endwin();
 printf("\n");
 exit(0);
};
